
-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_cameratoblender"

-- **************************************************
-- General information about this script
-- **************************************************

DS_cameratoblender = {}
function DS_cameratoblender:Name()
	return "Export camera to Blender"
end

function DS_cameratoblender:Version()
	return "0.1"
end

function DS_cameratoblender:Description()
	return "Export camera data to Blender"
end

function DS_cameratoblender:Creator()
	return "dauid.com"
end

function DS_cameratoblender:UILabel()
	return("Export camera to Blender")
end


-- **************************************************
-- The guts of this script
-- **************************************************


function DS_cameratoblender:OnMouseDown(moho, mouseEvent)
	self:Run(moho)
end

function DS_cameratoblender:Run(moho)

	local path = LM.GUI.SaveFile("Select Data File")
		if (path == "") then
			return
		end
		
			local f = io.open(path..".py", "w")

		if (f == nil) then
			return
		end

	local width = moho.document:Width()
	local height = moho.document:Height()
	local aspect = moho.document:AspectRatio()
	local newX = (moho.document:Width()/2)
	local newY = moho.document:Height()/2
	local newZ = moho.document:Height()/2


	local fps = moho.document:Fps()
	local startframe = moho.document:StartFrame()
	local endframe = moho.document:EndFrame()
	local campos= LM.Vector3:new_local()
	local pantilt = LM.Vector2:new_local()
	f:write("import bpy\n")
	f:write("cameras = {} \n")
	f:write("scene = bpy.context.scene\n")
	
	f:write("# Set in and out point \n")
	f:write("scene.frame_start =  ".. startframe .."\n")
	f:write("scene.frame_end =  ".. endframe .."\n")
	f:write("scene.render.fps =  ".. fps .."\n")
	f:write("scene.render.resolution_x =  ".. width .."\n")
	f:write("scene.render.resolution_y =  ".. height .."\n")
	
	
	f:write("frame = scene.frame_current - 1\n")
	f:write("data = bpy.data.cameras.new('Camera')\n")
	f:write("\n")--1/(tan(FOV/2)
	f:write("data.lens = ".. 1/math.tan(math.rad(60/moho.document.fCameraZoom:GetValue(0))/2)*9.8439 .."\n")

	--f:write("data.lens = ".. 9.8/math.tan(math.rad(60*moho.document.fCameraZoom:GetValue(0))/2) .."\n")
	--f:write("data.lens = ".. (62.22/2)*math.tan(moho.document.fCameraZoom:GetValue(0)*60) .."\n")
	--f:write("data.lens = ".. moho.document.fCameraZoom:GetValue(0)*17 .."\n")
	f:write("data.sensor_width = ".. 35 .."\n")
	f:write("data.shift_x = 0 \n")
	f:write("data.shift_y = 0 \n")
	f:write("data.dof_distance = 0.0 \n")
	f:write("data.clip_start = 0.1 \n")
	f:write("data.clip_end = 100 \n")
	f:write("data.draw_size = 0.5 \n")
	f:write("obj = bpy.data.objects.new('Camera', data) \n")
	f:write("obj.hide_render = False \n")
	f:write("scene.objects.link(obj) \n")
	f:write("cameras['Camera'] = obj \n")
	f:write("\n")
	
	
	local numlayers = moho.document:CountLayers() --Do top layers
		for l = 0, numlayers - 1 do
			local layer = moho.document:Layer(l)
			f:write("bpy.ops.object.add(type='EMPTY') \n")
			f:write("bpy.context.active_object.name = '".. layer:Name() ..".".. l .."' \n")
				local translation = layer.fTranslation:GetValue(0)
				local anchor = layer:Origin()
				translation.x = (translation.x+anchor.x)
				translation.y = (translation.y+anchor.y)
			f:write("bpy.data.objects['".. layer:Name() ..".".. l .."'].location = ".. translation.x ..", ".. translation.y ..", ".. translation.z .." \n")
	end
	


	f:write("\n")

	for frame = startframe, endframe do

		f:write("# new frame\n")
		f:write("scene.frame_set(1 + ".. frame ..")\n")
		f:write("obj = cameras['Camera']\n")
			local campos = moho.document.fCameraTrack:GetValue(frame)
		f:write("obj.location = ".. campos.x ..", ".. campos.y ..", ".. campos.z .." \n")
		f:write("obj.scale = 1, 1, 1 \n")
			local rotz = moho.document.fCameraRoll:GetValue(frame)
			local pantilt = moho.document.fCameraPanTilt:GetValue(frame)
		f:write("obj.rotation_euler = ".. pantilt.y*-1 ..", ".. pantilt.x ..", ".. rotz*(-1) .." \n")
		f:write("obj.keyframe_insert('location') \n")
		f:write("obj.keyframe_insert('scale') \n")
		f:write("obj.keyframe_insert('rotation_euler') \n")
		f:write("data = obj.data \n")
			local zoom = moho.document.fCameraZoom:GetValue(frame)
		f:write("data.lens = ".. 1/math.tan(math.rad(60/moho.document.fCameraZoom:GetValue(frame))/2)*9.8439 .."\n")
		--f:write("data.lens = ".. 9.8/math.tan(math.rad(moho.document.fCameraZoom:GetValue(frame))/2) .."\n")
		--f:write("data.lens = ".. zoom*17 .." \n")
		f:write("data.keyframe_insert('lens') \n")
		f:write("\n")
	end
	
	
f:close() --Close the file
end