-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_Curvature_Tool"

-- **************************************************
-- General information about this script
-- **************************************************

DS_Curvature_Tool = {}


function DS_Curvature_Tool:Name()
	return "DS Curvature Tool"
end

function DS_Curvature_Tool:Version()
	return "0.1"
end

function DS_Curvature_Tool:Description()
	return "Description"
end

function DS_Curvature_Tool:Creator()
	return "dauid.com"
end

function DS_Curvature_Tool:UILabel()
	return "Curvature tool"
end

-- **************************************************
-- Recurring values
-- **************************************************

DS_Curvature_Tool.numSel = 0
DS_Curvature_Tool.selID = -1
DS_Curvature_Tool.savedVal = 0
DS_Curvature_Tool.ExtraSmooth = false

-- **************************************************
-- The guts of this script
-- **************************************************

function DS_Curvature_Tool:IsEnabled(moho)
	if (moho:CountPoints() > 0) then
		return true
	end
	return false
end

function DS_Curvature_Tool:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.selID = -1
	self.selList = MOHO.SelectedPointList(mesh)
	self.numSel = table.getn(self.selList)

	if (self.numSel < 2) then -- work with only the closest selection
		-- find the closest point here
		self.selID = mesh:ClosestPoint(mouseEvent.startVec)
		if (self.selID >= 0) then
			self.numSel = 1
			mesh:SelectNone()
			local pt = mesh:Point(self.selID)
			pt.fSelected = true
		end
			
		local curve = nil
		local ptPos = -1
		
		if (self.numSel == 1) then
		local pt = mesh:Point(self.selID)
			for j = 0, pt:CountCurves() - 1 do
				curve, ptPos = pt:Curve(j, ptPos)
				local c = curve:GetCurvature(ptPos, moho.layerFrame)
				if (c>0.01) then
				pt:SetCurvature(MOHO.PEAKED, moho.layerFrame)
				else
					if (self.ExtraSmooth==false) then
					pt:SetCurvature(MOHO.SMOOTH, moho.layerFrame)
					else
					pt:SetCurvature(0.6667, moho.layerFrame)
					end
				end
			end
			
			moho:UpdateSelectedChannels()
		end
		
	end

	self.savedVal = 0
	mouseEvent.view:DrawMe()
end

function DS_Curvature_Tool:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local newVal = (mouseEvent.pt.x - mouseEvent.startPt.x) / mouseEvent.view:Graphics():Width()
	local curve = nil
	local ptPos = -1

	if (self.numSel == 1) then
		local pt = mesh:Point(self.selID)
		for j = 0, pt:CountCurves() - 1 do
			curve, ptPos = pt:Curve(j, ptPos)
			local c = curve:GetCurvature(ptPos, moho.layerFrame)
			c = c - self.savedVal + newVal
			c = LM.Clamp(c, MOHO.PEAKED, 2 / 3)
			curve:SetCurvature(ptPos, c, moho.layerFrame)
		end
	else
		for i, pt in ipairs(self.selList) do
			for j = 0, pt:CountCurves() - 1 do
				curve, ptPos = pt:Curve(j, ptPos)
				local c = curve:GetCurvature(ptPos, moho.layerFrame)
				c = c - self.savedVal + newVal
				c = LM.Clamp(c, MOHO.PEAKED, 2 / 3)
				curve:SetCurvature(ptPos, c, moho.layerFrame)
			end
		end
	end

	self.savedVal = newVal
	mouseEvent.view:DrawMe()
end

function DS_Curvature_Tool:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho:NewKeyframe(CHANNEL_CURVE)
	self.selList = nil
end

function DS_Curvature_Tool:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

function DS_Curvature_Tool:KeyAll(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
	local curve = nil
	local ptPos = -1
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
	
		for i = 0, mesh:CountPoints() - 1 do
			local pt = mesh:Point(i)
			for j = 0, pt:CountCurves() - 1 do
				curve, ptPos = pt:Curve(j, ptPos)
				local c = curve:GetCurvature(ptPos, moho.layerFrame)
					pt:SetCurvature(c, moho.layerFrame)
			end
			moho:NewKeyframe(CHANNEL_CURVE)
			moho:UpdateSelectedChannels()
		end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

DS_Curvature_Tool.PEAK = MOHO.MSG_BASE
DS_Curvature_Tool.SMOOTH = MOHO.MSG_BASE + 1
DS_Curvature_Tool.DUMMY = MOHO.MSG_BASE + 2
DS_Curvature_Tool.SELECTITEM = MOHO.MSG_BASE + 3
DS_Curvature_Tool.EXTRASMOOTH = MOHO.MSG_BASE + 4
DS_Curvature_Tool.KEYALL = MOHO.MSG_BASE + 5

function DS_Curvature_Tool:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu(MOHO.Localize("/Scripts/Tool/Curvature/SelectGroup=Select Group"))

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	self.peak = LM.GUI.ImageButton("ScriptResources/peak", MOHO.Localize("/Scripts/Tool/Peak/Peak=Peak"), false, self.PEAK)
	layout:AddChild(self.peak)

	self.smooth = LM.GUI.ImageButton("ScriptResources/smooth", MOHO.Localize("/Scripts/Tool/Peak/Smooth=Smooth"), false, self.SMOOTH)
	layout:AddChild(self.smooth)
	
	self.extrasmoothcheck = LM.GUI.CheckBox("Full smooth", self.EXTRASMOOTH)
	self.extrasmoothcheck:SetValue(false)
	layout:AddChild(self.extrasmoothcheck)
	
	self.keyallBut = LM.GUI.Button("Key all", self.KEYALL)
	layout:AddChild(self.keyallBut)
end

function DS_Curvature_Tool:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)
end

function DS_Curvature_Tool:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.PEAK) then
		local mesh = moho:Mesh()
		if (mesh == nil) then
			return
		end
	
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
	
		for i = 0, mesh:CountPoints() - 1 do
			local pt = mesh:Point(i)
			if (pt.fSelected) then
				pt:SetCurvature(MOHO.PEAKED, moho.layerFrame)
			end
		end
		moho:NewKeyframe(CHANNEL_CURVE)
	elseif (msg == self.SMOOTH) then
		local mesh = moho:Mesh()
		if (mesh == nil) then
			return
		end
	
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
	
		for i = 0, mesh:CountPoints() - 1 do
			local pt = mesh:Point(i)
			if (pt.fSelected) then
				pt:SetCurvature(MOHO.SMOOTH, moho.layerFrame)
			end
		end
		moho:NewKeyframe(CHANNEL_CURVE)
		
		elseif (msg == self.EXTRASMOOTH) then
		self.ExtraSmooth=self.extrasmoothcheck:Value()
		elseif (msg == self.KEYALL) then
		self:KeyAll(moho)
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()

	
	end
end


