-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_InstantLimbTool"

-- **************************************************
-- General information about this script
-- **************************************************

DS_InstantLimbTool = {}
function DS_InstantLimbTool:Name()
	return "Instant Limb Tool"
end

function DS_InstantLimbTool:Version()
	return "0.9"
end

function DS_InstantLimbTool:Description()
	return "Creates a limb ready to animate."
end

function DS_InstantLimbTool:Creator()
	return "dauid.com"
end

function DS_InstantLimbTool:UILabel()
	return("Instant Limb Tool")
end
-- **************************************************
-- Recurring values
-- **************************************************
DS_InstantLimbTool.parentbone = nil						--The parent of the selected bone
DS_InstantLimbTool.lowerbone = nil						--The selected bone
DS_InstantLimbTool.anglebone = nil						--The bone that will be added at the joint
DS_InstantLimbTool.upperAngle =0 						--This will store the angle of the upper bone
DS_InstantLimbTool.middleAngle = 0 						--This will store the angle of the added bone, i.e half the lower bone's angle
DS_InstantLimbTool.lowerAngle=0 						--This will store the angle of the lower bone
DS_InstantLimbTool.upperPos = LM.Vector2:new_local()	--Position of the upper bone
DS_InstantLimbTool.lowerPos = LM.Vector2:new_local()	--Position of the lower bone
DS_InstantLimbTool.thirdPos = LM.Vector2:new_local()	--The end point of the lower bone
DS_InstantLimbTool.jointCurvature=0.4406054019928		--Line curvature in the elbow/knee
DS_InstantLimbTool.upperJoint = LM.Vector2:new_local() 	--Position of the upper cross section of the arm
DS_InstantLimbTool.lowerJoint = LM.Vector2:new_local()	--Position of the lower cross section of the arm
DS_InstantLimbTool.bentWrongWay=false					--If the lower bone is rotated clockwise at an angle greater than the upper bone then the points in the very middle of the arm have to be flipped.
DS_InstantLimbTool.lowerShapeOnTop=true					--True for making an arm (the lower arm will overlap the upper arm. False for making a leg ( not necessarily but I think it looks best).
DS_InstantLimbTool.lineWidth=0.0055555556900799			--The line width and also radius of the gap filling circles
DS_InstantLimbTool.fillGaps=true						--If true will create circles to fill line gaps in joints
DS_InstantLimbTool.upperRadius=0.15						--Radius of shoulder
DS_InstantLimbTool.lowerRadius=0.15						--Radius of elbow
DS_InstantLimbTool.bottomRadius=0.15					--Radius of wrist
DS_InstantLimbTool.theDistance = 0.025					--The distance between cross sections in the arm
DS_InstantLimbTool.lineWidthInMoho=0					--For calculating line width	
DS_InstantLimbTool.mesh=nil	
DS_InstantLimbTool.oldNumPoints=0
-- **************************************************
-- The guts of this script
-- **************************************************

function DS_InstantLimbTool:IsEnabled(moho)
if (moho.layer:LayerType() ~= MOHO.LT_VECTOR) then
		return false
	else
		return true
	end
end


function DS_InstantLimbTool:OnMouseDown(moho, mouseEvent)

--Resetting some stuff to avoid errors
DS_InstantLimbTool.parentbone = nil
DS_InstantLimbTool.lowerbone = nil
DS_InstantLimbTool.anglebone = nil
DS_InstantLimbTool.upperPos = LM.Vector2:new_local()
DS_InstantLimbTool.lowerPos = LM.Vector2:new_local()
DS_InstantLimbTool.thirdPos = LM.Vector2:new_local()
DS_InstantLimbTool.upperJoint = LM.Vector2:new_local()
DS_InstantLimbTool.lowerJoint = LM.Vector2:new_local()



local skel = moho:ParentSkeleton()
	if (skel == nil) then
		LM.GUI.Alert(LM.GUI.ALERT_INFO, "No parent skeleton found.", nil, nil, "OK", nil, nil)
		return
	end

for i = 0, skel:CountBones() - 1 do --Go through every bone
		if (skel:Bone(i).fSelected) then --If the current bone is selected
		local tempb=skel:Bone(i).fParent
			if (tempb<0) then
				LM.GUI.Alert(LM.GUI.ALERT_INFO, "Bone does not have a parent.", "Please select a child bone.", nil, "OK", nil, nil)
				return
			end
		end
end

if (moho.document:Height()>moho.document:Width()) then
		DS_InstantLimbTool.lineWidthInMoho=moho.document:Width()
		else
		DS_InstantLimbTool.lineWidthInMoho=moho.document:Height()
		end

self.mesh = moho:Mesh()
if (self.mesh == nil) then
		return
	end

moho.document:PrepUndo(moho.layer)
moho.document:SetDirty()
-------------------------------------Get info about bone positions and angles
self.bentWrongWay=false
self.upperAngle=0
self.upperPos:Set(0, 0)
	
	for lower = 0, skel:CountBones() - 1 do --Go through every bone
	 self.lowerbone = skel:Bone(lower)
		if (self.lowerbone.fSelected) then --If the current bone is selected
			break
			end
	end
			
	if (self.lowerbone:Name()=="") then
	self.lowerbone:SetName("Random Name"..math.random(9999))
	end
		
	self.parentbone=skel:Bone(self.lowerbone.fParent) --Get parent bone of selected bone

--------Bunch of  code to get the positions and rotations of any parent bones to the upper bone in the limb (Big thanks to HeyVern at the Lost Marble forum)
		if (self.parentbone.fParent>-1) then 
		
            local bParent = self.lowerbone.fParent 
            while bParent >-1 do 
                local bone = skel:Bone(bParent) 
                self.upperAngle = self.upperAngle + bone.fAngle 
                bParent = bone.fParent 
            end 
            self.parentbone.fRestMatrix:Transform(self.upperPos)
		else
		self.upperPos=self.parentbone.fPos
		self.upperAngle=self.parentbone.fAngle
		end
---------------------------------------------------------------------------------------------------------------------------------

		if (self.lowerbone.fAngle>math.pi)then
		self.bentWrongWay=true
		end
			self.lowerAngle = self.upperAngle+self.lowerbone.fAngle --Get the angle
			self.middleAngle = self.upperAngle+(self.lowerbone.fAngle*0.5)
			self.lowerPos=self.parentbone.fPos+self.lowerbone.fPos 
			self.lowerPos.x=self.upperPos.x+math.cos(self.upperAngle)*(self.lowerbone.fPos.x) --Get the position of what I call middle joint
			self.lowerPos.y=self.upperPos.y+math.sin(self.upperAngle)*(self.lowerbone.fPos.x)
	
			self.thirdPos.x=self.lowerPos.x+math.cos(self.lowerAngle)*(self.lowerbone.fLength)
			self.thirdPos.y=self.lowerPos.y+math.sin(self.lowerAngle)*(self.lowerbone.fLength)
			
-------------------------------------End of info gathering
self:BuildLimb(moho)
moho.view:DrawMe()

end

function DS_InstantLimbTool:buildMesh(moho)

local mesh = self.mesh
	if (mesh == nil) then
		return
	end

local v = LM.Vector2:new_local()
DS_InstantLimbTool.oldNumPoints=mesh:CountPoints()
local oldNumPoints = mesh:CountPoints()

self.upperJoint.x=self.upperPos.x+math.cos(self.upperAngle)*(self.lowerbone.fPos.x-self.theDistance) --Get the position of what I call the upper joint
self.upperJoint.y=self.upperPos.y+math.sin(self.upperAngle)*(self.lowerbone.fPos.x-self.theDistance)
		
self.lowerJoint.x=self.lowerPos.x+math.cos(self.lowerAngle)*(self.theDistance) --then get the poition offset on the self.lowerbone. Hard to explain
self.lowerJoint.y=self.lowerPos.y+math.sin(self.lowerAngle)*(self.theDistance)

v.x=(self.upperPos.x)+(math.cos(self.upperAngle+(math.pi/2))*self.upperRadius)
v.y=(self.upperPos.y)+(math.sin(self.upperAngle+(math.pi/2))*self.upperRadius)
local pointOne=(mesh:AddLonePoint(v, 0))
v.x=(self.upperPos.x)+(math.cos(self.upperAngle+math.pi)*self.upperRadius)
v.y=(self.upperPos.y)+(math.sin(self.upperAngle+math.pi)*self.upperRadius)
local pointTwo=(mesh:AppendPoint(v, 0))
v.x=(self.upperPos.x)+(math.cos(self.upperAngle+(math.pi*1.5))*self.upperRadius)
v.y=(self.upperPos.y)+(math.sin(self.upperAngle+(math.pi*1.5))*self.upperRadius)
local pointThree=(mesh:AppendPoint(v, 0))

v.x=(self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi*1.5))*self.lowerRadius)
v.y=(self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi*1.5))*self.lowerRadius)
local pointFour=(mesh:AppendPoint(v, 0))
----------------------------------------------------------------------------------------
	if (self.bentWrongWay) then
		v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius)
		v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius)
	else
		v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
		v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	end
local pointFive=(mesh:AppendPoint(v, 0))
----------------------------------------------------------------------------------------
v.x=(self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi*1.5))*self.lowerRadius)
v.y=(self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi*1.5))*self.lowerRadius)
local pointSix=(mesh:AppendPoint(v, 0))

v.x=(self.thirdPos.x)+(math.cos(self.lowerAngle+(math.pi*1.5))*self.bottomRadius)
v.y=(self.thirdPos.y)+(math.sin(self.lowerAngle+(math.pi*1.5))*self.bottomRadius)
local pointSeven=(mesh:AppendPoint(v, 0))

v.x=self.thirdPos.x
v.y=self.thirdPos.y
local pointEight=(mesh:AppendPoint(v, 0))

v.x=(self.thirdPos.x)+(math.cos(self.lowerAngle+(math.pi/2))*self.bottomRadius)
v.y=(self.thirdPos.y)+(math.sin(self.lowerAngle+(math.pi/2))*self.bottomRadius)
local pointNine=(mesh:AppendPoint(v, 0))

v.x=(self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi/2))*self.lowerRadius)
v.y=(self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi/2))*self.lowerRadius)
local pointTen=(mesh:AppendPoint(v, 0))
---------------------------------------------------------------------------------------
	if (self.bentWrongWay) then
		v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
		v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	else
		v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius)
		v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius)
	end
local pointEleven=(mesh:AppendPoint(v, 0))
---------------------------------------------------------------------------------------
v.x=(self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi/2))*self.lowerRadius)
v.y=(self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi/2))*self.lowerRadius)
local pointTwelve=(mesh:AppendPoint(v, 0))

v.x=(self.upperPos.x)+(math.cos(self.upperAngle+(math.pi/2))*self.upperRadius)
v.y=(self.upperPos.y)+(math.sin(self.upperAngle+(math.pi/2))*self.upperRadius)
local pointThirteen=(mesh:AppendPoint(v, 0))

mesh:WeldPoints(oldNumPoints, oldNumPoints+12, 0) --Weld the big limb shape
--------------------------------------------------------------------------------------------- Create upper cross section
mesh:SelectNone()
v.x=(self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi*1.5))*self.lowerRadius)
v.y=(self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi*1.5))*self.lowerRadius)
local pointFourteen=(mesh:AddLonePoint(v, 0))

v.x=(self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi/2))*self.lowerRadius)
v.y=(self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi/2))*self.lowerRadius)
local pointFifteen=(mesh:AppendPoint(v, 0))
mesh:SelectNone()

mesh:WeldPoints(oldNumPoints+2, oldNumPoints+12, 0) --Weld the upper cross section
mesh:WeldPoints(oldNumPoints+12, oldNumPoints+9, 0) --Weld the upper cross section
--------------------------------------------------------------------------------------------- Create middle cross section
mesh:SelectNone()
	if (self.bentWrongWay) then
	v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius)
	v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius)
	else
	v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	end
local pointSixteen=(mesh:AddLonePoint(v, 0))
	
	if (self.bentWrongWay) then
	v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	else
	v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius)
	v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius)
	end

local pointSeventeen=(mesh:AppendPoint(v, 0))

mesh:WeldPoints(oldNumPoints+2, oldNumPoints+12, 0) --Weld the middle cross section
mesh:WeldPoints(oldNumPoints+12, oldNumPoints+7, 0) --Weld the middle cross section
--------------------------------------------------------------------------------------------- Create middle cross section
mesh:SelectNone()
v.x=(self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi*1.5))*self.lowerRadius)
v.y=(self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi*1.5))*self.lowerRadius)
local pointEighteen=(mesh:AddLonePoint(v, 0))

v.x=(self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi/2))*self.lowerRadius)
v.y=(self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi/2))*self.lowerRadius)
local pointNineteen=(mesh:AppendPoint(v, 0))

mesh:WeldPoints(oldNumPoints+2, oldNumPoints+12, 0) --Weld the lower cross section
mesh:WeldPoints(oldNumPoints+12, oldNumPoints+5, 0) --Weld the lower cross section
--------------------------------------------------------------------------------------------- Set the curvature
local peakedTable= {oldNumPoints+1, oldNumPoints+2, oldNumPoints+4, oldNumPoints+5, oldNumPoints+7, oldNumPoints+8, oldNumPoints+9, oldNumPoints+11}
for i, v in ipairs(peakedTable) do
	mesh:Point(v):SetCurvature(MOHO.PEAKED, 0)
end

mesh:Point(oldNumPoints):SetCurvature(0.65895956754684, 0)
mesh:Point(oldNumPoints+3):SetCurvature(0.65895956754684, 0)

mesh:Point(oldNumPoints+6):SetCurvature(self.jointCurvature, 0)
mesh:Point(oldNumPoints+10):SetCurvature(self.jointCurvature, 0)

mesh:SelectAll()



end
------------------------------------------------------End of buildMesh
function DS_InstantLimbTool:BuildLimb(moho)
local skel = moho:ParentSkeleton()
	if (skel == nil) then
		return
	end

	
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
------------------------------------------------------------------------------------------------ Add the extra bone
	self.angleBone=skel:AddBone(0)
	local NumBones=skel:CountBones()-1
	skel:Bone(NumBones).fParent=skel:BoneID(self.parentbone)
	skel:Bone(NumBones).fAnimPos:SetValue(0, self.lowerbone.fPos)
	skel:Bone(NumBones).fAnimAngle:SetValue(0, (math.pi*1.5)+(self.lowerbone.fAngle/2))
	skel:Bone(NumBones).fLength=self.lowerRadius
	skel:Bone(NumBones).fAngleControlParent=skel:BoneID(self.lowerbone)
	skel:Bone(NumBones).fAngleControlScale=0.5
------------------------------------------------------------------------------------------------ Start building
mesh:SelectNone()
local oldNumPoints = mesh:CountPoints()
local oldNumCurves = mesh:CountCurves()
DS_InstantLimbTool:buildMesh(mesh)
mesh:SelectNone()
--------------------------------------------------------------------------------------------- Hide edges
mesh:Curve(oldNumCurves+1):SetSegmentOn(0, false)
mesh:Curve(oldNumCurves+2):SetSegmentOn(0, false)
mesh:Curve(oldNumCurves+3):SetSegmentOn(0, false)
--------------------------------------------------------------------------------------------- Fill gaps with circles

if (self.fillGaps) then
	if (self.bentWrongWay) then
			self:fillGapFunction(mesh, (self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius), (self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius), self.lineWidth)
			self:fillGapFunction(mesh, (self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius), (self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius), self.lineWidth)
		else
			self:fillGapFunction(mesh, (self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius), (self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius), self.lineWidth)
			self:fillGapFunction(mesh, (self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius), (self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius), self.lineWidth)
		end

self:fillGapFunction(mesh, (self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi*1.5))*self.lowerRadius), (self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi*1.5))*self.lowerRadius), self.lineWidth)
self:fillGapFunction(mesh, (self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi/2))*self.lowerRadius), (self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi/2))*self.lowerRadius), self.lineWidth)
self:fillGapFunction(mesh, (self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi*1.5))*self.lowerRadius), (self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi*1.5))*self.lowerRadius), self.lineWidth)
self:fillGapFunction(mesh, (self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi/2))*self.lowerRadius), (self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi/2))*self.lowerRadius), self.lineWidth)
end

--------------------------------------------------------------------------------------------- Start making shapes
local upperShape={oldNumPoints, oldNumPoints+1, oldNumPoints+7, oldNumPoints+8, oldNumPoints+9}
local upperMiddleShape = {oldNumPoints+6, oldNumPoints+7, oldNumPoints+9, oldNumPoints+10}
local lowerMiddleShape = {oldNumPoints+5, oldNumPoints+6, oldNumPoints+10, oldNumPoints+11}
local lowerShape = {oldNumPoints+2, oldNumPoints+3, oldNumPoints+4, oldNumPoints+5, oldNumPoints+11}
local topCircles = {oldNumPoints+20, oldNumPoints+21, oldNumPoints+22, oldNumPoints+23, oldNumPoints+24, oldNumPoints+25, oldNumPoints+26, oldNumPoints+27}
local middleCircles = {oldNumPoints+12, oldNumPoints+13, oldNumPoints+14, oldNumPoints+15, oldNumPoints+16, oldNumPoints+17, oldNumPoints+18, oldNumPoints+19}
local bottomCircles = {oldNumPoints+28, oldNumPoints+29, oldNumPoints+30, oldNumPoints+31, oldNumPoints+32, oldNumPoints+33, oldNumPoints+34, oldNumPoints+35}

	if (self.lowerShapeOnTop) then
	
		for i, v in ipairs(upperShape) do
			mesh:Point(v).fSelected=true
		end
		local theUpperShape=moho:CreateShape(true)
		mesh:Shape(theUpperShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		for i, v in ipairs(upperMiddleShape) do
			mesh:Point(v).fSelected=true
		end
		local theUpperMiddleShape=moho:CreateShape(true)
		mesh:Shape(theUpperMiddleShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(topCircles) do
			mesh:Point(v).fSelected=true
			end
			local theTopCircles=moho:CreateShape(true)
			
			mesh:Shape(theTopCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theTopCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theTopCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			
			mesh:SelectNone()
		end
		
		for i, v in ipairs(lowerMiddleShape) do
			mesh:Point(v).fSelected=true
		end
		local theLowerMiddleShape=moho:CreateShape(true)
		mesh:Shape(theLowerMiddleShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(middleCircles) do
			mesh:Point(v).fSelected=true
			end
			local theMiddleCircles=moho:CreateShape(true)
			
			mesh:Shape(theMiddleCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theMiddleCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theMiddleCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			mesh:SelectNone()
		end
		
		for i, v in ipairs(lowerShape) do
			mesh:Point(v).fSelected=true
		end
		local theLowerShape=moho:CreateShape(true)
		mesh:Shape(theLowerShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(bottomCircles) do
			mesh:Point(v).fSelected=true
			end
			local theBottomCircles=moho:CreateShape(true)
			
			mesh:Shape(theBottomCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theBottomCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theBottomCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			
			mesh:SelectNone()
		end
		
	else
	
		for i, v in ipairs(lowerShape) do
				mesh:Point(v).fSelected=true
			end
			local theLowerShape=moho:CreateShape(true)
			mesh:Shape(theLowerShape).fMyStyle.fLineWidth=self.lineWidth
			mesh:SelectNone()
			
		for i, v in ipairs(lowerMiddleShape) do
			mesh:Point(v).fSelected=true
		end
		local theLowerMiddleShape=moho:CreateShape(true)
		mesh:Shape(theLowerMiddleShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(bottomCircles) do
			mesh:Point(v).fSelected=true
			end
			local theBottomCircles=moho:CreateShape(true)
			
			mesh:Shape(theBottomCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theBottomCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theBottomCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			
			mesh:SelectNone()
		end
		
		for i, v in ipairs(upperMiddleShape) do
			mesh:Point(v).fSelected=true
		end
		local theUpperMiddleShape=moho:CreateShape(true)
		mesh:Shape(theUpperMiddleShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(middleCircles) do
			mesh:Point(v).fSelected=true
			end
			local theMiddleCircles=moho:CreateShape(true)
			
			mesh:Shape(theMiddleCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theMiddleCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theMiddleCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			mesh:SelectNone()
		end
		
		for i, v in ipairs(upperShape) do
			mesh:Point(v).fSelected=true
		end
		local theUpperShape=moho:CreateShape(true)
		mesh:Shape(theUpperShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(topCircles) do
			mesh:Point(v).fSelected=true
			end
			local theTopCircles=moho:CreateShape(true)
			
			mesh:Shape(theTopCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theTopCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theTopCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			
			mesh:SelectNone()
		end
	end
--------------------------------------------------------------------------------------------- Bind to bones

	local upperBoneBinding = {oldNumPoints, oldNumPoints+1, oldNumPoints+7, oldNumPoints+8, oldNumPoints+9}

			for i, v in ipairs(upperBoneBinding) do
				mesh:Point(v).fParent=(skel:BoneID(self.parentbone))
			end

	local middleBoneBinding = {oldNumPoints+6, oldNumPoints+10}

			for i, v in ipairs(middleBoneBinding) do
				mesh:Point(v).fParent=(NumBones)
			end
	local lowerboneBinding = {oldNumPoints+2, oldNumPoints+3, oldNumPoints+4, oldNumPoints+5, oldNumPoints+11}

			for i, v in ipairs(lowerboneBinding) do
				mesh:Point(v).fParent=(skel:BoneID(self.lowerbone))
			end
		
	if (self.fillGaps) then
		for i, v in ipairs(topCircles) do
				mesh:Point(v).fParent=(skel:BoneID(self.parentbone))
		end
		
		for i, v in ipairs(middleCircles) do
				mesh:Point(v).fParent=(NumBones)
		end
		
		for i, v in ipairs(bottomCircles) do
				mesh:Point(v).fParent=(skel:BoneID(self.lowerbone))
		end
	end
	

-------------------------------------------------------------------------------
	moho.view:DrawMe()
end

function DS_InstantLimbTool:fillGapFunction(mesh, theX, theY, size)


local oldNumPoints = mesh:CountPoints()
	local v = LM.Vector2:new_local()
	mesh:SelectNone()
	
	
	v.x=theX-size
	v.y=theY
local pointOne=(mesh:AddLonePoint(v, 0))
	v.x=theX
	v.y=theY+size
local pointTwo=(mesh:AppendPoint(v, 0))

	v.x=theX+size
	v.y=theY
local pointThree=(mesh:AppendPoint(v, 0))

	v.x=theX
	v.y=theY-size
local pointFour=(mesh:AppendPoint(v, 0))

	v.x=theX-size
	v.y=theY
local pointFive=(mesh:AppendPoint(v, 0))

mesh:WeldPoints(oldNumPoints, oldNumPoints+4, 0)
	
	for i = 0, 3 do
		mesh:Point(oldNumPoints+i):SetCurvature(0.391379, 0)
		mesh:Point(oldNumPoints+i).fSelected=true
		
	end
	mesh:SelectNone()
end

function DS_InstantLimbTool:DeletePoints(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
	
	if (DS_InstantLimbTool.fillGaps) then
	for i = 1, 29  do
			mesh:DeletePoint(self.oldNumPoints)

		end
	else
		for i = 1, 11  do
			mesh:DeletePoint(self.oldNumPoints)

		end
	end
end


function DS_InstantLimbTool:GatherFunctions(moho)

	self:DeletePoints(moho)
	self:BuildLimb(moho)
		

end

function DS_InstantLimbTool:DelBone(moho)
local skel = moho:ParentSkeleton()
	if (skel == nil) then
		return
	end
	skel:DeleteBone(skel:CountBones()-1, 0)
end


-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

DS_InstantLimbTool.UPPERR = MOHO.MSG_BASE
DS_InstantLimbTool.MIDDLER = MOHO.MSG_BASE + 1
DS_InstantLimbTool.BOTTOMR = MOHO.MSG_BASE + 2
DS_InstantLimbTool.LINEWIDTH = MOHO.MSG_BASE + 3
DS_InstantLimbTool.CURVATURE = MOHO.MSG_BASE + 4
DS_InstantLimbTool.DISTANCE = MOHO.MSG_BASE + 5
DS_InstantLimbTool.UPPERSHAPE = MOHO.MSG_BASE + 6
DS_InstantLimbTool.LOWERSHAPE = MOHO.MSG_BASE + 7
DS_InstantLimbTool.FILLGAPS = MOHO.MSG_BASE + 8
DS_InstantLimbTool.CREATE = MOHO.MSG_BASE + 9

function DS_InstantLimbTool:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Upper radius:"))
	self.upper = LM.GUI.TextControl(0, "00.0000", self.UPPERR, LM.GUI.FIELD_UFLOAT)
	self.upper:SetWheelInc(0.05)
	self.upper:SetValue(self.upperRadius)
	layout:AddChild(self.upper)
	
	layout:AddChild(LM.GUI.StaticText("Middle:"))
	self.middle = LM.GUI.TextControl(0, "00.0000", self.MIDDLER, LM.GUI.FIELD_UFLOAT)
	self.middle:SetWheelInc(0.05)
	self.middle:SetValue(self.lowerRadius)
	layout:AddChild(self.middle)
	
	layout:AddChild(LM.GUI.StaticText("Bottom:"))
	self.bottom = LM.GUI.TextControl(0, "00.0000", self.BOTTOMR, LM.GUI.FIELD_UFLOAT)
	self.bottom:SetWheelInc(0.05)
	self.bottom:SetValue(self.bottomRadius)
	layout:AddChild(self.bottom)

	layout:AddChild(LM.GUI.StaticText("Line width:"))
	self.thelinewidth = LM.GUI.TextControl(0, "00.0000", self.LINEWIDTH, LM.GUI.FIELD_UFLOAT)
	self.thelinewidth:SetWheelInc(1)
	self.thelinewidth:SetValue(self.lineWidth*self.lineWidthInMoho)
	layout:AddChild(self.thelinewidth)
	
	layout:AddChild(LM.GUI.StaticText("Joint curvature:"))
	self.curvature = LM.GUI.TextControl(0, "00.0000", self.CURVATURE, LM.GUI.FIELD_UFLOAT)
	self.curvature:SetWheelInc(0.05)
	self.curvature:SetValue(self.jointCurvature)
	layout:AddChild(self.curvature)
	
	layout:AddChild(LM.GUI.StaticText("Section distance:"))
	self.distance = LM.GUI.TextControl(0, "00.0000", self.DISTANCE, LM.GUI.FIELD_UFLOAT)
	self.distance:SetWheelInc(0.05)
	self.distance:SetValue(self.theDistance)
	layout:AddChild(self.distance)
	
	self.lowerOnTop=LM.GUI.RadioButton("Lower shape on top", self.LOWERSHAPE)
	layout:AddChild(self.lowerOnTop, LM.GUI.ALIGN_LEFT)
	self.upperOnTop=LM.GUI.RadioButton("Upper shape on top", self.UPPERSHAPE)
	layout:AddChild(self.upperOnTop, LM.GUI.ALIGN_LEFT)
	
	if (self.lowerShapeOnTop) then
	self.lowerOnTop:SetValue(true)
	self.upperOnTop:SetValue(false)
	else
	self.lowerOnTop:SetValue(false)
	self.upperOnTop:SetValue(true)
	end
	
	self.fillbutton = LM.GUI.CheckBox("Fill gaps:", self.FILLGAPS)
	layout:AddChild(self.fillbutton)
	self.fillbutton:SetValue(self.fillGaps)

	
end

function DS_InstantLimbTool:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

end

function DS_InstantLimbTool:HandleMessage(moho, view, msg)


	if (msg == self.UPPERR) then
		
		self.upperRadius = self.upper:FloatValue()
		self:DelBone(moho)
		self:GatherFunctions(moho)
	end
	if (msg == self.MIDDLER) then
		self.lowerRadius = self.middle:FloatValue()
		self:DelBone(moho)
		self:GatherFunctions(moho)
	end
	if (msg == self.BOTTOMR) then
		self.bottomRadius = self.bottom:FloatValue()
		self:DelBone(moho)
		self:GatherFunctions(moho)
	end
	if (msg == self.LINEWIDTH) then
		self.lineWidth = (self.thelinewidth:FloatValue()/self.lineWidthInMoho)
		self:DelBone(moho)
		self:GatherFunctions(moho)
	end
	if (msg == self.CURVATURE) then
		self.jointCurvature = self.curvature:FloatValue()
		self:DelBone(moho)
		self:GatherFunctions(moho)
	end
	if (msg == self.DISTANCE) then
		self.theDistance = self.distance:FloatValue()
		self:DelBone(moho)
		self:GatherFunctions(moho)
	end
	
	if (msg == self.LOWERSHAPE) then
		self.lowerShapeOnTop = true
		self.lowerOnTop:SetValue(true)
		self.upperOnTop:SetValue(false)
		self:DelBone(moho)
		self:GatherFunctions(moho)
	end
	
	if (msg == self.UPPERSHAPE) then
		self.lowerShapeOnTop = false
		self.lowerOnTop:SetValue(false)
		self.upperOnTop:SetValue(true)
		self:DelBone(moho)
		self:GatherFunctions(moho)
	end
	
	if (msg == self.FILLGAPS) then
		self:DeletePoints(moho)
		self.fillGaps=self.fillbutton:Value()
		self:DelBone(moho)
		self:BuildLimb(moho)
	
	end
	
end
