-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_Pick_char"

-- **************************************************
-- General information about this script
-- **************************************************

DS_Pick_char = {}

DS_Pick_char.BASE_STR = 2370

function DS_Pick_char:Name()
	return "Translate Character"
end

function DS_Pick_char:Version()
	return "0.1"
end

function DS_Pick_char:IsBeginnerScript()
	return true
end

function DS_Pick_char:Description()
	return "Translate Character"
end

function DS_Pick_char:Creator()
	return "dauid.com"
end

function DS_Pick_char:UILabel()
	return"Translate Character"
end

function DS_Pick_char:LoadPrefs(prefs)
	self.displayOn = prefs:GetBool("DS_Pick_char.displayOn", true)
end

function DS_Pick_char:SavePrefs(prefs)
	prefs:SetBool("DS_Pick_char.displayOn", self.displayOn)
end

function DS_Pick_char:ResetPrefs()
	self.displayOn = true
end

-- **************************************************
-- Recurring values
-- **************************************************

DS_Pick_char.selCount = 1
DS_Pick_char.startVal = LM.Vector3:new_local()
DS_Pick_char.when = -10000
DS_Pick_char.TOLERANCE = 10
DS_Pick_char.savedpos = {}
DS_Pick_char.newpos = {}
DS_Pick_char.allKeys = false
DS_Pick_char.allActions = false
DS_Pick_char.prevStart = LM.Vector3:new_local()
DS_Pick_char.prevNew = LM.Vector3:new_local()
DS_Pick_char.moving = false

-- **************************************************
-- The guts of this script
-- **************************************************

function DS_Pick_char:OnMouseDown(moho, mouseEvent)

	


	
	
	local layer = mouseEvent.view:PickGlobalLayer(mouseEvent.pt)
	if layer ~= nil then
		--print(layer:Parent():Name())
		moho:SetSelLayer(layer:Parent()) 
		moho:UpdateUI()

	end
moho.document:PrepMultiUndo()
	moho.document:SetDirty()
	self.selCount = moho.document:CountSelectedLayers()
	self.when = -10000
	if (mouseEvent.ctrlKey) then -- Not sure if this should require a modifier key or not...
		self.when = -20000
		local g = mouseEvent.view:Graphics()
		local m = LM.Matrix:new_local()
		local vec = LM.Vector2:new_local()
		local origin = moho.layer:Origin()
		local pt = LM.Point:new_local()
		local totalTimingOffset = moho.layer:TotalTimingOffset()
		-- First see if any keyframes were picked
		for i = 0, moho.layer.fTranslation:CountKeys() - 1 do
			local frame = moho.layer.fTranslation:GetKeyWhen(i)
			moho.layer:GetFullTransform(frame - totalTimingOffset, m, moho.document)
			vec:Set(origin.x, origin.y)
			m:Transform(vec)
			if (moho.layer.fTranslation:HasKey(frame)) then
				g:WorldToScreen(vec, pt)
				if (math.abs(pt.x - mouseEvent.startPt.x) < self.TOLERANCE and math.abs(pt.y - mouseEvent.startPt.y) < self.TOLERANCE) then
					self.when = frame
					break
				end
			end
		end
		-- If no keyframes were picked, try picking a random point along the curve.
		if (self.when <= -10000) then
			local startFrame = moho.layer.fTranslation:GetKeyWhen(0)
			local endFrame = moho.layer.fTranslation:Duration()
			if (endFrame > startFrame) then
				local vec3 = LM.Vector3:new_local()
				local oldVec3 = LM.Vector3:new_local()
				g:Clear(0, 0, 0, 0)
				g:SetColor(255, 255, 255)
				g:BeginPicking(mouseEvent.startPt, 4)
				for frame = startFrame, endFrame do
					moho.layer:GetFullTransform(frame - totalTimingOffset, m, moho.document)
					vec3:Set(origin.x, origin.y, 0)
					m:Transform(vec3)
					if (frame > startFrame) then
						g:DrawLine(oldVec3.x, oldVec3.y, vec3.x, vec3.y)
					end
					if (g:Pick()) then
						self.when = frame
						break
					end
					oldVec3:Set(vec3)
				end
				if (self.when > -10000) then -- We picked a point on the curve.
					if (self.selCount > 1) then
						return
					end
					moho.layer.fTranslation:AddKey(self.when)
				end
			end
		end
	end

	if (self.when == -20000) then
		return
	end

	self.startVal = {}
	if (self.when > -10000) then
		local startVal = LM.Vector3:new_local()
		startVal:Set(moho.layer.fTranslation:GetValue(self.when))
		table.insert(self.startVal, startVal)
		-- Put the keyframe time in terms of document time, so that it works as expected during mouse dragging.
		self.when = self.when - moho.layer:TotalTimingOffset()
	else
		self.startVal = {}
		for i = 0, self.selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
			local startVal = LM.Vector3:new_local()
			startVal:Set(layer.fTranslation.value)
			table.insert(self.startVal, startVal)
			layer.fTranslation:AddKey(moho.frame + layer:TotalTimingOffset())
		end
	end
	
	
	if self.allKeys or self.allActions then
		self.prevStart = moho.layer.fTranslation:GetValue(moho.frame)
		self.savedpoints = {}
		for i = 0, self.selCount - 1 do
			self.savedpoints[i] = LM.Vector3:new_local()
			local layer = moho.document:GetSelectedLayer(i)
			self.savedpos[i]=layer.fTranslation:GetValue(moho.frame)
		end
	end

	mouseEvent.view:DrawMe()
end

function DS_Pick_char:OnMouseMoved(moho, mouseEvent)
	if (self.when == -20000) then
		return
	end
	local frame = self.when
	if (frame > -10000 and self.selCount > 1) then
		return
	end
	if (frame <= -10000) then
		frame = moho.frame
	end
	
	self.moving = true

	if (not moho.layer:IsImmuneToCamera()) then
		local v1 = LM.Vector3:new_local()
		local v2 = LM.Vector3:new_local()
		local vec = LM.Vector3:new_local()
		local m = LM.Matrix:new_local()

		for i = 0, self.selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)

			m:Identity()
			if (layer:Parent()) then
				layer:Parent():GetFullTransform(moho.frame, m, moho.document)
			elseif (moho.document:IsOutsideViewEnabled()) then
				moho.document:GetOutsideViewMatrix(m)
			else
				moho.document:GetCameraMatrix(moho.frame, m)
			end
			if (layer:LayerParentBone() ~= -1 and layer:Parent() and layer:Parent():IsBoneType()) then
				local parentM = LM.Matrix:new_local()
				layer:GetParentBoneTransform(moho.frame, parentM, moho.document)
				m:Multiply(parentM)
			end
			m:Invert()
			mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.startPt, v1)
			mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.pt, v2)
	
			vec.x = v2.x - v1.x
			vec.y = v2.y - v1.y
	
			if (mouseEvent.altKey) then
				v2.z = v1.z - (v2.y - v1.y) * 0.25
				v2.x = 0.0
				v1.x = 0.0
				v2.y = 0.0
				v1.y = 0.0
			else
				if (mouseEvent.shiftKey) then
					if (math.abs(vec.x) > math.abs(vec.y)) then
						v2.y = v1.y
					else
						v2.x = v1.x
					end
				end
			end

			v1.z = v1.z - 0.885 ---0.95
			v2.z = v2.z - 0.885 ---0.95

			m:Transform(v1)
			m:Transform(v2)
	
			vec = self.startVal[i + 1] + v2 - v1
			layer.fTranslation:SetValue(frame + layer:TotalTimingOffset(), vec)
		end -- for i
	else
		for i = 0, self.selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
			layer.fTranslation:SetValue(frame + layer:TotalTimingOffset(), self.startVal[i + 1])

			local layerM = LM.Matrix:new_local()
			local tempLayer = MOHO.MohoLayer:new_local()
			tempLayer.fTranslation:SetValue(0, layer.fTranslation.value)

			if (layer:Parent()) then
				local parentM = LM.Matrix:new_local()
				tempLayer:GetFullTransform(moho.frame, layerM, nil)
				layer:Parent():GetFullTransform(moho.frame, parentM, moho.document)
				layerM:Multiply(parentM)
				if (layer:LayerParentBone() ~= -1 and layer:Parent():IsBoneType()) then
					layer:GetParentBoneTransform(moho.frame, parentM, moho.document)
					layerM:Multiply(parentM)
				end
			elseif (layer:IsImmuneToCamera()) then
				tempLayer:GetFullTransform(moho.frame, layerM, nil)
			else
				tempLayer:GetFullTransform(moho.frame, layerM, moho.document)
			end
			self.startVec = mouseEvent.view:Point2Vec(mouseEvent.startPt, layerM)
			self.nextVec = mouseEvent.view:Point2Vec(mouseEvent.pt, layerM)
	
			local vec = LM.Vector3:new_local()

			if (mouseEvent.altKey) then
				mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.startPt, self.startVec)
				mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.pt, self.nextVec)
				vec:Set(self.startVal[i + 1])
				vec.z = self.startVal[i + 1].z - (self.nextVec.y - self.startVec.y)
				layer.fTranslation:SetValue(frame + layer:TotalTimingOffset(), vec)
	
				if (layer:Parent()) then
					layer:Parent():DepthSort(moho.document)
				end
			else
				vec.x = self.nextVec.x - self.startVec.x
				vec.y = self.nextVec.y - self.startVec.y
	
				if (mouseEvent.shiftKey) then
					if (math.abs(vec.x) > math.abs(vec.y)) then
						vec.y = 0
					else
						vec.x = 0
					end
				end

				vec = vec + self.startVal[i + 1]
				layer.fTranslation:SetValue(frame + layer:TotalTimingOffset(), vec)
			end
		end -- for i
	end

	if (frame ~= moho.frame) then
		moho:SetCurFrame(moho.frame) -- force a refresh when editing a key at a different frame
	end
	if self.allKeys then
		self.prevNew = moho.layer.fTranslation:GetValue(moho.frame)
	end

	moho.document:DepthSort()
	mouseEvent.view:DrawMe()
end

function DS_Pick_char:OnMouseUp(moho, mouseEvent)
	if (self.when == -2) then
		return
	end

	if (self.when < 0) then
		moho:NewKeyframe(CHANNEL_LAYER_T)
	end
	self.moving = false
	
	for i = 0, self.selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
	
	
	if self.allKeys then
	--self.prevNew = moho.layer.fTranslation:GetValue(moho.frame)
		for i = 0, self.selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
			local newv = layer.fTranslation:GetValue(moho.frame)
			--print(self.savedpoints[i].x)
				local ch = layer.fTranslation
				for f = 0, ch:Duration() do
					if f~=moho.frame then
						if ch:HasKey(f) then
							local currv = layer.fTranslation:GetValue(f)
							--print(self.savedpos[i].x)
							layer.fTranslation:SetValue(f, currv+(newv-self.savedpos[i]))
						end
					end
				end
			end
		end	
	end
	
	moho:UpdateUI()
end

function DS_Pick_char:OnKeyDown(moho, keyEvent)
	if (keyEvent.ctrlKey) then
		local inc = 1
		if (keyEvent.shiftKey) then
			inc = 10
		end

		local m = LM.Matrix:new_local()
		moho.layer:GetFullTransform(moho.frame, m, moho.document)

		local fakeME = {}
		fakeME.view = keyEvent.view
		fakeME.pt = LM.Point:new_local()
		fakeME.pt:Set(keyEvent.view:Graphics():Width() / 2, keyEvent.view:Graphics():Height() / 2)
		fakeME.startPt = LM.Point:new_local()
		fakeME.startPt:Set(fakeME.pt)
		fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
		fakeME.startVec = keyEvent.view:Point2Vec(fakeME.pt, m)
		fakeME.shiftKey = false
		fakeME.ctrlKey = false
		fakeME.altKey = keyEvent.altKey
		fakeME.penPressure = 0

		if (keyEvent.keyCode == LM.GUI.KEY_UP) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.y = fakeME.pt.y - inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.y = fakeME.pt.y + inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_LEFT) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.x = fakeME.pt.x - inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_RIGHT) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.x = fakeME.pt.x + inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		end
	end
end

function DS_Pick_char:OnInputDeviceEvent(moho, deviceEvent)
	if (deviceEvent.inputData:GetString("DeviceType") == "Wacom Multitouch") then
		local mtState = deviceEvent.inputData:GetInt("MultitouchState")
		local mtMousePt = deviceEvent.inputData:GetPoint("MultitouchCenterPoint")
		if (mtState == 1) then -- first finger down
			self.mtFingersTouching = 0 -- we'll get the correct number on the next event
			self.mtTranslate = LM.Point:new_local()
			self.mtStartPt = deviceEvent.inputData:GetPoint("MultitouchCenterPoint")
			self.mtAccumTranslate = LM.Point:new_local()
			self.mtScale = 1.0
			self.mtStartScale = 1.0
			self.mtAccumScale = 1.0
			self.mtAngle = 0.0
			self.mtStartAngle = 0.0
			self.mtAccumAngle = 0.0
			
			self.selCount = moho.document:CountSelectedLayers()
			self.when = -10000
			
			moho.document:SetDirty()
			moho.document:PrepMultiUndo()
			
			self.startVal = {}
			self.startAngle = {}
			self.startScale = {}
			for i = 0, self.selCount - 1 do
				local layer = moho.document:GetSelectedLayer(i)
				local startVal = LM.Vector3:new_local()
				startVal:Set(layer.fTranslation.value)
				table.insert(self.startVal, startVal)
				layer.fTranslation:AddKey(moho.frame + layer:TotalTimingOffset())
				table.insert(self.startAngle, layer.fRotationZ.value)
				startVal = LM.Vector3:new_local()
				startVal:Set(layer.fScale.value)
				table.insert(self.startScale, startVal)
			end
			
			self.mtTranslate:Set(0.0, 0.0)
			self.mtScale = 1.0
			self.mtAngle = 0.0
		elseif (mtState == 3) then -- last finger up
			self.mtFingersTouching = 0
			if (self.when < 0) then
				moho:NewKeyframe(CHANNEL_LAYER_T)
			end
			moho:UpdateUI()
		elseif (mtState == 2) then -- dragging
			local fingersTouching = deviceEvent.inputData:GetInt("FingersTouching")
			if (fingersTouching ~= self.mtFingersTouching) then
				self.mtAccumTranslate = self.mtAccumTranslate + self.mtTranslate
				self.mtAccumScale = self.mtAccumScale * self.mtScale
				self.mtAccumAngle = self.mtAccumAngle + self.mtAngle
				self.mtFingersTouching = fingersTouching
				self.mtStartPt = deviceEvent.inputData:GetPoint("MultitouchCenterPoint")
				self.mtStartAngle = deviceEvent.inputData:GetFloat("MultitouchAngle")
				self.mtStartScale = deviceEvent.inputData:GetFloat("MultitouchScale")
			end
			
			self.mtTranslate = mtMousePt - self.mtStartPt
			self.mtScale = 1.0
			self.mtAngle = 0.0
			if (self.mtFingersTouching > 1) then
				self.mtAngle = deviceEvent.inputData:GetFloat("MultitouchAngle") - self.mtStartAngle
				self.mtScale = deviceEvent.inputData:GetFloat("MultitouchScale") / self.mtStartScale
			end
			
			local frame = self.when
			if (frame <= -10000) then
				frame = moho.frame
			end
			
			if (not moho.layer:IsImmuneToCamera()) then
				local v1 = LM.Vector3:new_local()
				local v2 = LM.Vector3:new_local()
				local vec = LM.Vector3:new_local()
				local m = LM.Matrix:new_local()
		
				for i = 0, self.selCount - 1 do
					local layer = moho.document:GetSelectedLayer(i)
		
					m:Identity()
					if (layer:Parent()) then
						layer:Parent():GetFullTransform(moho.frame, m, moho.document)
					elseif (moho.document:IsOutsideViewEnabled()) then
						moho.document:GetOutsideViewMatrix(m)
					else
						moho.document:GetCameraMatrix(moho.frame, m)
					end
					if (layer:LayerParentBone() ~= -1 and layer:Parent() and layer:Parent():IsBoneType()) then
						local parentM = LM.Matrix:new_local()
						layer:GetParentBoneTransform(moho.frame, parentM, moho.document)
						m:Multiply(parentM)
					end
					m:Invert()
					deviceEvent.view:Graphics():ScreenToWorld(self.mtStartPt, v1)
					deviceEvent.view:Graphics():ScreenToWorld(mtMousePt + self.mtAccumTranslate, v2)
			
					vec.x = v2.x - v1.x
					vec.y = v2.y - v1.y
		
					if (deviceEvent.altKey) then
						v2.z = v1.z - (v2.y - v1.y) * 0.25
						v2.x = 0.0
						v1.x = 0.0
						v2.y = 0.0
						v1.y = 0.0
					else
						if (deviceEvent.shiftKey) then
							if (math.abs(vec.x) > math.abs(vec.y)) then
								v2.y = v1.y
							else
								v2.x = v1.x
							end
						end
					end
					
					v1.z = v1.z - 0.885 ---0.95
					v2.z = v2.z - 0.885 ---0.95
		
					m:Transform(v1)
					m:Transform(v2)
			
					vec = self.startVal[i + 1] + v2 - v1
					layer.fTranslation:SetValue(frame + layer:TotalTimingOffset(), vec)
					if (self.mtFingersTouching > 1) then
						layer.fRotationZ:SetValue(frame + layer:TotalTimingOffset(), self.startAngle[i + 1] + self.mtAccumAngle + self.mtAngle)
						layer.fScale:SetValue(frame + layer:TotalTimingOffset(), self.startScale[i + 1] * self.mtAccumScale * self.mtScale)
					end
				end -- for i
			else
				for i = 0, self.selCount - 1 do
					local layer = moho.document:GetSelectedLayer(i)
					layer.fTranslation:SetValue(frame + layer:TotalTimingOffset(), self.startVal[i + 1])
		
					local layerM = LM.Matrix:new_local()
					local tempLayer = MOHO.MohoLayer:new_local()
					tempLayer.fTranslation:SetValue(0, layer.fTranslation.value)
		
					if (layer:Parent()) then
						local parentM = LM.Matrix:new_local()
						tempLayer:GetFullTransform(moho.frame, layerM, nil)
						layer:Parent():GetFullTransform(moho.frame, parentM, moho.document)
						layerM:Multiply(parentM)
						if (layer:LayerParentBone() ~= -1 and layer:Parent():IsBoneType()) then
							layer:GetParentBoneTransform(moho.frame, parentM, moho.document)
							layerM:Multiply(parentM)
						end
					elseif (layer:IsImmuneToCamera()) then
						tempLayer:GetFullTransform(moho.frame, layerM, nil)
					else
						tempLayer:GetFullTransform(moho.frame, layerM, moho.document)
					end
					self.startVec = deviceEvent.view:Point2Vec(self.mtStartPt, layerM)
					self.nextVec = deviceEvent.view:Point2Vec(mtMousePt + self.mtAccumTranslate, layerM)
			
					local vec = LM.Vector3:new_local()
		
					if (deviceEvent.altKey) then
						deviceEvent.view:Graphics():ScreenToWorld(self.mtStartPt, self.startVec)
						deviceEvent.view:Graphics():ScreenToWorld(mtMousePt, self.nextVec)
						vec:Set(self.startVal[i + 1])
						vec.z = self.startVal[i + 1].z - (self.nextVec.y - self.startVec.y)
						layer.fTranslation:SetValue(frame + layer:TotalTimingOffset(), vec)
			
						if (layer:Parent()) then
							layer:Parent():DepthSort(moho.document)
						end
					else
						vec.x = self.nextVec.x - self.startVec.x
						vec.y = self.nextVec.y - self.startVec.y
						
						if (deviceEvent.shiftKey) then
							if (math.abs(vec.x) > math.abs(vec.y)) then
								vec.y = 0
							else
								vec.x = 0
							end
						end
		
						vec = vec + self.startVal[i + 1]
						layer.fTranslation:SetValue(frame + layer:TotalTimingOffset(), vec)
					end
	
					vec = vec + self.startVal[i + 1]
					layer.fTranslation:SetValue(frame + layer:TotalTimingOffset(), vec)
					if (self.mtFingersTouching > 1) then
						layer.fRotationZ:SetValue(frame + layer:TotalTimingOffset(), self.startAngle[i + 1] + self.mtAccumAngle + self.mtAngle)
						layer.fScale:SetValue(frame + layer:TotalTimingOffset(), self.startScale[i + 1] * self.mtAccumScale * self.mtScale)
					end
				end -- for i
			end
			
			if (frame ~= moho.frame) then
				moho:SetCurFrame(moho.frame) -- force a refresh when editing a key at a different frame
			end
			moho.document:DepthSort()
		end

		return true
	end

	return false
end

function DS_Pick_char:findKeys(moho)
	local ch = moho.layer.fTranslation
	local frame = moho.frame
	local earlierFrame, laterFrame
	for f = frame, 0, -1 do
		if ch:HasKey(f) then
			earlierFrame = f
			break
		end
	end
	for f = frame, ch:Duration() do
		if ch:HasKey(f) then
			laterFrame = f
			break
		end
	end
	
	return earlierFrame, laterFrame
end

function DS_Pick_char:DrawMe(moho, view, compensate)
	if (not self.displayOn) then
		return
	end
	if (moho.layer:PhysicsParent(-1) ~= nil) then
		return
	end

	local startFrame = moho.layer.fTranslation:GetKeyWhen(0)
	local endFrame = moho.layer.fTranslation:Duration()

	if (endFrame > startFrame) then
		local g = view:Graphics()
		local m = LM.Matrix:new_local()
		local vec = LM.Vector3:new_local()
		local oldVec = LM.Vector3:new_local()
		local origin = moho.layer:Origin()
		local totalTimingOffset = moho.layer:TotalTimingOffset()

		g:SetColor(102, 152, 203)
		g:SetSmoothing(true)
		
		for frame = startFrame, endFrame do
			moho.layer:GetFullTransform(frame - totalTimingOffset, m, moho.document)
			local earlierFrame, laterFrame
			vec:Set(origin.x, origin.y, 0)
			if self.allKeys and frame ~= moho.frame and self.moving then
				earlierFrame, laterFrame = self:findKeys(moho)
				vec = vec + (self.prevNew-self.prevStart)
			else
				earlierFrame, laterFrame = frame, frame
			end
			m:Transform(vec)
			if (frame > startFrame) then
				if self.allKeys then
					if not self.moving then
						g:DrawLine(oldVec.x, oldVec.y, vec.x, vec.y)
					end
				else
				g:DrawLine(oldVec.x, oldVec.y, vec.x, vec.y)
				end
			end
			if (moho.layer.fTranslation:HasKey(frame)) then
				g:DrawFatMarker(vec.x, vec.y, 3)
			end
			oldVec:Set(vec)
		end
		g:SetSmoothing(false)
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

DS_Pick_char.CHANGE = MOHO.MSG_BASE
DS_Pick_char.RESET = MOHO.MSG_BASE + 1
DS_Pick_char.TOGGLE_DISPLAY = MOHO.MSG_BASE + 2
DS_Pick_char.FLIP_H = MOHO.MSG_BASE + 3
DS_Pick_char.FLIP_V = MOHO.MSG_BASE + 4
DS_Pick_char.ALLKEYS = MOHO.MSG_BASE + 5
DS_Pick_char.ALLACTIONS = MOHO.MSG_BASE + 6
DS_Pick_char.displayOn = true

function DS_Pick_char:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/TranslateLayer/Position=Position")))

	layout:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/TranslateLayer/X=X:")))
	self.textX = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textX)

	layout:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/TranslateLayer/Y=Y:")))
	self.textY = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textY)

	layout:AddChild(LM.GUI.StaticText(MOHO.Localize("/Scripts/Tool/TranslateLayer/Z=Z:")))
	self.textZ = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textZ)

	layout:AddChild(LM.GUI.Button(MOHO.Localize("/Scripts/Tool/TranslateLayer/Reset=Reset"), self.RESET))

	self.displayCheck = LM.GUI.CheckBox(MOHO.Localize("/Scripts/Tool/TranslateLayer/ShowPath=Show path"), self.TOGGLE_DISPLAY)
	layout:AddChild(self.displayCheck)

	layout:AddChild(LM.GUI.ImageButton("ScriptResources/flip_layer_h", MOHO.Localize("/Scripts/Tool/SetOrigin/FlipH=Flip Layer Horizontally"), false, self.FLIP_H))
	layout:AddChild(LM.GUI.ImageButton("ScriptResources/flip_layer_v", MOHO.Localize("/Scripts/Tool/SetOrigin/FlipV=Flip Layer Vertically"), false, self.FLIP_V))
	
	self.allKeysbox = LM.GUI.CheckBox("Change all keys relatively", self.ALLKEYS)
	layout:AddChild(self.allKeysbox)
	
	self.allActionsbox = LM.GUI.CheckBox("Change all actions relatively", self.ALLACTIONS)
	--layout:AddChild(self.allActionsbox)
end

function DS_Pick_char:UpdateWidgets(moho)
	self.textX:SetValue(moho.layer.fTranslation.value.x)
	self.textY:SetValue(moho.layer.fTranslation.value.y)
	self.textZ:SetValue(moho.layer.fTranslation.value.z)
	self.displayCheck:SetValue(self.displayOn)
	self.allKeysbox:SetValue(self.allKeys)
	self.allActionsbox:SetValue(self.allActions)
end

function DS_Pick_char:HandleMessage(moho, view, msg)
	local newVal = LM.Vector3:new_local()
	local selCount = moho.document:CountSelectedLayers()

	if (msg == self.RESET) then
		moho.document:PrepMultiUndo()
		moho.document:SetDirty()

		for i = 0, selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
			if (moho.frame == 0) then
				newVal:Set(0, 0, 0)
				layer.fTranslation:SetValue(0, newVal)
			else
				newVal:Set(layer.fTranslation:GetValue(0))
				layer.fTranslation:SetValue(moho.frame + layer:TotalTimingOffset(), newVal)
			end
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_LAYER_T)
		moho.document:DepthSort()
	elseif (msg == self.CHANGE) then
		moho.document:PrepMultiUndo()
		moho.document:SetDirty()

		newVal.x = self.textX:FloatValue()
		newVal.y = self.textY:FloatValue()
		newVal.z = self.textZ:FloatValue()
		for i = 0, selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
			local vec = layer.fTranslation:GetValue(moho.frame + layer:TotalTimingOffset()) - newVal
			if (vec:Mag() > 0.0001) then
				layer.fTranslation:SetValue(moho.frame + layer:TotalTimingOffset(), newVal)
				moho:NewKeyframe(CHANNEL_LAYER_T)
			end
		end
		moho.document:DepthSort()
	elseif (msg == self.ALLKEYS) then
		self.allKeys = self.allKeysbox:Value()
	elseif (msg == self.ALLACTIONS) then
		self.allActions = self.allActionsbox:Value()
	elseif (msg == self.TOGGLE_DISPLAY) then
		self.displayOn = self.displayCheck:Value()
	elseif (msg == self.FLIP_H) then
		moho.document:PrepMultiUndo()
		moho.document:SetDirty()
		for i = 0, selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
			layer.fFlipH:SetValue(moho.frame + layer:TotalTimingOffset(), not layer.fFlipH.value)
			moho:NewKeyframe(CHANNEL_LAYER_FLIP_H)
		end
	elseif (msg == self.FLIP_V) then
		moho.document:PrepMultiUndo()
		moho.document:SetDirty()
		for i = 0, selCount - 1 do
			local layer = moho.document:GetSelectedLayer(i)
			layer.fFlipV:SetValue(moho.frame + layer:TotalTimingOffset(), not layer.fFlipV.value)
			moho:NewKeyframe(CHANNEL_LAYER_FLIP_V)
		end
	end
end
